use std::{error::Error, fs::{self, read_dir, DirBuilder, File}, io::Write, path::Path};

use ascii::AsciiString;
use tiny_http::{Response, Server};


mod handlers;
mod structs;
pub mod blog_entries;
pub mod rand_quote;



fn main() {
    let output_path = "output";
    match DirBuilder::new()
        .recursive(true)
        .create(output_path) {
        Ok(_) => (),
        Err(_) => (),
    }

    match copy_dir_all("assets", format!("{output_path}/assets")) {
        Ok(_) => (),
        Err(err) => println!("Couldnt copy assets directory, err: {err}"),
    }



    fs::write(format!("{output_path}/index.html"), handlers::index().as_bytes()).expect("Couldnt write index file");
    fs::write(format!("{output_path}/about.html"), handlers::about().as_bytes()).expect("Couldnt write about file");
    fs::write(format!("{output_path}/404.html"), handlers::not_found().as_bytes()).expect("Couldnt write 404 file");

    match DirBuilder::new()
        .create(format!("{output_path}/blog")) {
        Ok(_) => (),
        Err(_) => (),
    }


    let post_dir_iter = match read_dir("posts/") {
        Ok(iter) => iter,
        Err(err) => panic!("could ls files, err {err}")
    };

    for entry in post_dir_iter {
        if let Ok(entry) = entry {

            let filename = entry.file_name().into_string().unwrap().split(".").collect::<Vec<_>>()[0].to_string();

            fs::write(format!("{output_path}/blog/{filename}.html"), handlers::blog(filename).as_bytes()).expect("Couldnt write blog entry");
        };

    }

}
fn copy_dir_all(src: impl AsRef<Path>, dst: impl AsRef<Path>) -> std::io::Result<()> {
    std::fs::create_dir_all(&dst)?;
    for entry in std::fs::read_dir(src)? {
        let entry = entry?;
        let ty = entry.file_type()?;
        if ty.is_dir() {
            copy_dir_all(entry.path(), dst.as_ref().join(entry.file_name()))?;
        } else {
            std::fs::copy(entry.path(), dst.as_ref().join(entry.file_name()))?;
        }
    }
    Ok(())
}

fn old_main() {

    //    let app: axum::Router = Router::new()
    //        .route("/", get(handlers::index))
    //        .route("/about", get(handlers::about))
    //        .route("/blog",get(handlers::index))
    //        .route("/blog/:blog_path",get(handlers::blog))
    //        .route_service("/robots.txt", ServeFile::new("assets/robots.txt"))
    //        .fallback(get(handlers::not_found))
    //    .nest_service("/assets", ServeDir::new("assets"));
    //
    //
    //    let listener = tokio::net::TcpListener::bind("127.0.0.1:3000").await.unwrap();
    //    axum::serve(listener, app).await.unwrap();
    //
    //-----
    //    let server = Arc::new(Server::http("127.0.0.1:3001").unwrap());
    //    println!("started server on port 3001");
    //
    //    let mut handles = Vec::new();
    //
    //    for thread_num in 0..6 {
    //        println!("starting thread {}",thread_num);
    //        let server = server.clone();
    //        handles.push(thread::spawn(move || server_thread(server)));
    //
    //    }
    //    for h in handles {
    //        h.join().unwrap();
    //    }
    // ------------
    let server = Server::http("127.0.0.1:3000").unwrap();
    println!("started server on port 3001");
    server_thread(server);

}

fn server_thread(server: Server) {
    for request in server.incoming_requests() {
        println!("received request; method: {}, url: {}",
            request.method(),request.url());

        match request.url() {
            "" | "/" => {
                let response = Response::from_string(handlers::index());
                let response = response.with_header(tiny_http::Header {
                    field: "Content-Type".parse().unwrap(),
                    value: AsciiString::from_ascii("text/html;charset=utf8").unwrap(),
                });

                let _ = request.respond(response);

            }
            "/about" => {
                let response = Response::from_string(handlers::about());
                let response = response.with_header(tiny_http::Header {
                    field: "Content-Type".parse().unwrap(),
                    value: AsciiString::from_ascii("text/html;charset=utf8").unwrap(),
                });

                let _ = request.respond(response);

            }
            asset_url @ _ if asset_url.starts_with("/assets/") => {
                println!("getting asset : {}",asset_url);
                if let Some(asset) = get_asset(&asset_url){
                    let _ = request.respond(asset);
                }

            }
            blog_url @ _ if blog_url.starts_with("/blog/") => {


                let response = Response::from_string(handlers::blog(blog_url[6..].to_string()));
                let response = response.with_header(tiny_http::Header {
                    field: "Content-Type".parse().unwrap(),
                    value: AsciiString::from_ascii("text/html;charset=utf8").unwrap(),
                });

                let _ = request.respond(response);

            }
            &_ => {

                let response = Response::from_string(handlers::not_found());
                let response = response.with_header(tiny_http::Header {
                    field: "Content-Type".parse().unwrap(),
                    value: AsciiString::from_ascii("text/html;charset=utf8").unwrap(),
                });

                let _ = request.respond(response);
            }
        }
    }
}



fn get_asset(asset:&str)-> Option<Response<File>>{

    match Path::new(&asset[1..]).exists() {
        true => {
            println!("found asset");
            Some(Response::from_file(File::open(&asset[1..]).unwrap()))
        }
        false => None,
    }

}










